/**********************************************************************
 *
 * librarymain.cpp -- 
 * Copyright (C) 1999  The New Zealand Digital Library Project
 *
 * A component of the Greenstone digital library software
 * from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: librarymain.cpp 963 2000-02-21 22:01:02Z sjboddie $
 *
 *********************************************************************/

/*
   $Log$
   Revision 1.23  2000/02/21 21:59:37  sjboddie
   gsdlhome now comes from gsdlsite.cfg

   Revision 1.22  2000/02/17 02:35:48  sjboddie
   tidied up a bit

   Revision 1.21  2000/02/15 22:53:51  kjm18
   search history stuff added.

   Revision 1.20  1999/12/13 02:46:56  davidb
   Teach buildaction about recptionist

   Revision 1.19  1999/11/01 21:58:49  sjboddie
   no longer set receptionist for some actions that used to

   Revision 1.18  1999/10/25 22:29:31  sjboddie
   receptionist now checks collect diectory for collections
   rather than collections.txt file

   Revision 1.17  1999/10/19 03:23:43  davidb
   Collection building support through web pages
   and internal and external link handling for collection documents

   Revision 1.16  1999/10/18 20:07:33  sjboddie
   added htmlbrowserclass

   Revision 1.15  1999/10/14 23:03:10  sjboddie
   another browser

   Revision 1.14  1999/10/10 08:14:09  sjboddie
   - metadata now returns mp rather than array
   - redesigned browsing support (although it's not finished so
   won't currently work ;-)

   Revision 1.13  1999/09/16 21:39:35  sjboddie
   added gb converters

   Revision 1.12  1999/09/14 22:42:06  sjboddie
   included utf8 converter

   Revision 1.11  1999/09/14 22:03:27  sjboddie
   now reads in collections from a file to avoid recompiling

   Revision 1.10  1999/09/08 00:51:11  sjboddie
   removed old interface stuff

   Revision 1.9  1999/09/07 04:56:56  sjboddie
   added GPL notice

   Revision 1.8  1999/07/15 06:08:55  rjmcnab
   Moved the adding of the actions to librarymain so that they can
   be overriden easier.

   Revision 1.7  1999/06/15 01:56:12  sjboddie
   Got multiple collections working

   Revision 1.6  1999/05/10 03:40:39  sjboddie
   lots of changes - slowly getting document action sorted out

   Revision 1.5  1999/04/19 23:56:10  rjmcnab
   Finished the gdbm metadata stuff

   Revision 1.4  1999/04/12 03:45:05  rjmcnab
   Finished the query filter.

   Revision 1.3  1999/04/06 22:20:35  rjmcnab
   Got browsefilter working.

   Revision 1.2  1999/03/05 03:53:54  sjboddie

   fixed some bugs

   Revision 1.1  1999/02/21 22:35:22  rjmcnab

   Initial revision.

 */


#include "receptionist.h"
#include "cgiwrapper.h"
#include "nullproto.h"
#include "collectserver.h"
#include "filter.h"
#include "browsefilter.h"
#include "queryfilter.h"
#include "infodbclass.h"
#include "mgsearch.h"
#include "mggdbmsource.h"
#include "fileutil.h"
#include <assert.h>

#include "action.h"
#include "statusaction.h"
#include "pageaction.h"
#include "pingaction.h"
#include "queryaction.h"
#include "documentaction.h"
#include "authenaction.h"
#include "usersaction.h"
#include "authenaction.h"
#include "extlinkaction.h"
#include "buildaction.h"
#include "delhistoryaction.h"

#include "browserclass.h"
#include "vlistbrowserclass.h"
#include "hlistbrowserclass.h"
#include "datelistbrowserclass.h"
#include "invbrowserclass.h"
#include "pagedbrowserclass.h"
#include "htmlbrowserclass.h"

#include "recptconfig.h"

int main () {
  receptionist recpt;
  nullproto nproto;
  text_tarray collections;

  // get gsdlhome (if we fail the error will be picked up later -- in
  // cgiwrapper)
  text_t gsdlhome;
  if (site_cfg_read (gsdlhome)) {
    text_t collectdir = filename_cat (gsdlhome, "collect");
    if (!read_dir (collectdir, collections)) exit (1);
  }

  text_tarray::const_iterator thiscol = collections.begin();
  text_tarray::const_iterator endcol = collections.end();
  
  while (thiscol != endcol) {
    
    // ignore the modelcol
    if (*thiscol == "modelcol") {thiscol ++; continue;}
    
    // this memory is created but never destroyed
    // we're also not doing any error checking to make sure we didn't
    // run out of memory
    collectserver *cserver = new collectserver();
    gdbmclass *gdbmhandler = new gdbmclass();
    mgsearchclass *mgsearch = new mgsearchclass();
    
    // add a null filter
    filterclass *filter = new filterclass ();
    cserver->add_filter (filter);
      
    // add a browse filter
    browsefilterclass *browsefilter = new browsefilterclass();
    browsefilter->set_gdbmptr (gdbmhandler);
    cserver->add_filter (browsefilter);
    
    // add a query filter
    queryfilterclass *queryfilter = new queryfilterclass();
    queryfilter->set_gdbmptr (gdbmhandler);
    queryfilter->set_mgsearchptr (mgsearch);
    cserver->add_filter (queryfilter);
    
    // add a mg and gdbm source
    mggdbmsourceclass *mggdbmsource = new mggdbmsourceclass ();
    mggdbmsource->set_gdbmptr (gdbmhandler);
    mggdbmsource->set_mgsearchptr (mgsearch);
    cserver->add_source (mggdbmsource);
    
    // inform collection server and everything it contains about its
    // collection name
    cserver->configure ("collection", *thiscol);
    
    nproto.add_collectserver (cserver);
    thiscol ++;
  }
  
  // add the protocol to the receptionist
  recpt.add_protocol (&nproto);
  
  // add other converters
  utf8inconvertclass utf8inconvert;
  utf8outconvertclass utf8outconvert;
  recpt.add_converter ("u", &utf8inconvert, &utf8outconvert);

  if (!gsdlhome.empty()) {
    mapinconvertclass gbinconvert;
    gbinconvert.setmapfile (gsdlhome, "gbku", 0x25a1);
    mapoutconvertclass gboutconvert;
    gboutconvert.setmapfile (gsdlhome, "ugbk", 0xa1f5);
    recpt.add_converter ("g", &gbinconvert, &gboutconvert);
  }

  // the list of actions. Note: these actions will become invalid
  // at the end of this function.
  statusaction astatusaction;
  astatusaction.set_receptionist (&recpt);
  recpt.add_action (&astatusaction);

  pageaction apageaction;
  recpt.add_action (&apageaction);

  pingaction apingaction;
  recpt.add_action (&apingaction);

  queryaction aqueryaction;
  recpt.add_action (&aqueryaction);

  documentaction adocumentaction;
  recpt.add_action (&adocumentaction);

  usersaction ausersaction;
  recpt.add_action (&ausersaction);

  extlinkaction anextlinkaction;
  recpt.add_action (&anextlinkaction);

  buildaction abuildaction;
  abuildaction.set_receptionist (&recpt);
  recpt.add_action (&abuildaction);

  authenaction aauthenaction;
  aauthenaction.set_receptionist(&recpt);
  recpt.add_action (&aauthenaction);

  delhistoryaction adelhistoryaction;
  recpt.add_action(&adelhistoryaction);

  // list of browsers
  vlistbrowserclass avlistbrowserclass;
  recpt.add_browser (&avlistbrowserclass);
  recpt.setdefaultbrowser ("VList");

  hlistbrowserclass ahlistbrowserclass;
  recpt.add_browser (&ahlistbrowserclass);

  datelistbrowserclass adatelistbrowserclass;
  recpt.add_browser (&adatelistbrowserclass);

  invbrowserclass ainvbrowserclass;
  recpt.add_browser (&ainvbrowserclass);

  pagedbrowserclass apagedbrowserclass;
  recpt.add_browser (&apagedbrowserclass);

  htmlbrowserclass ahtmlbrowserclass;
  recpt.add_browser (&ahtmlbrowserclass);

  cgiwrapper (recpt, "");
  return 0;
}





