###########################################################################
#
# win32 makefile -- src/recpt
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the
# University of Waikato, New Zealand.
#
# Copyright (C) 1999  The New Zealand Digital Library Project
#
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################

AR = lib
CC = cl
CPPFLAGS =
DEFS = -DNZDL -DQUIET -D__WIN32__ -D_LITTLE_ENDIAN -DGSDL_NAMESPACE_BROKEN \
       -DSHORT_SUFFIX -DPARADOCNUM -DHAVE_CONFIG_H -D__STL_NO_NEW_IOSTREAMS \
       -DGSDLSERVER -DGSDLUSAGELOG	
INCLUDES = -I..\..\lib -I. -I..\colservr -I..\..\packages\mg \
	   -I..\..\packages\mg\lib -I..\..\packages\mg\src\text \
	   -I..\..\packages\wingdbm -If:\STLport-3.2\stlport \
	   -If:\cw32crypt

LDFLAGS =
LIBS = ..\..\packages\wingdbm\gdbm.lib f:\cw32crypt\crypt.lib

COMPILE = $(CC) -c $(CPPFLAGS) $(DEFS) $(INCLUDES)
LINK = $(CC) $(LDFLAGS) -o $@

.SUFFIXES:
.SUFFIXES: .c .obj
.c.obj:
	$(COMPILE) $<

.SUFFIXES: .cpp .obj
.cpp.obj:
	$(COMPILE) $< 

COMMONHEADERS = receptionist.h cgiwrapper.h cgiargs.h action.h browserclass.h \
                converter.h recptconfig.h cgiutils.h OIDtools.h browsetools.h htmlgen.h \
		statusaction.h pageaction.h comtypes.h recptproto.h \
		pingaction.h queryaction.h documentaction.h \
		extlinkaction.h buildaction.h htmlutils.h \
		querytools.h formattools.h tipaction.h authenaction.h infodbclass.h \
                userdb.h usersaction.h vlistbrowserclass.h hlistbrowserclass.h \
		datelistbrowserclass.h invbrowserclass.h pagedbrowserclass.h \
		htmlbrowserclass.h

RECEPTHEADERS = 

LIBRARYHEADERS = nullproto.h

HEADERS = $(COMMONHEADERS) $(RECPTHEADERS) $(LIBRARYHEADERS)


COMMONSOURCES = receptionist.cpp cgiwrapper.cpp cgiargs.cpp action.cpp browserclass.cpp \
                converter.cpp recptconfig.cpp cgiutils.cpp OIDtools.cpp browsetools.cpp \
		htmlgen.cpp statusaction.cpp pageaction.cpp comtypes.cpp \
		recptproto.cpp pingaction.cpp queryaction.cpp documentaction.cpp \
		extlinkaction.cpp buildaction.cpp htmlutils.cpp querytools.cpp formattools.cpp \
		tipaction.cpp authenaction.cpp infodbclass.cpp userdb.cpp usersaction.cpp \
		vlistbrowserclass.cpp hlistbrowserclass.cpp datelistbrowserclass.cpp \
		invbrowserclass.cpp pagedbrowserclass.cpp htmlbrowserclass.cpp


RECPTSOURCES = recptmain.cpp

LIBRARYSOURCES = nullproto.cpp librarymain.cpp

SOURCES = $(COMMONSOURCES) $(RECPTSOURCES) $(LIBRARYSOURCES)


COMMONOBJECTS = receptionist.obj cgiwrapper.obj cgiargs.obj action.obj browserclass.obj \
                converter.obj recptconfig.obj cgiutils.obj OIDtools.obj browsetools.obj \
		htmlgen.obj statusaction.obj pageaction.obj comtypes.obj \
		recptproto.obj pingaction.obj queryaction.obj documentaction.obj \
		extlinkaction.obj buildaction.obj htmlutils.obj querytools.obj \
		formattools.obj tipaction.obj authenaction.obj \
                infodbclass.obj userdb.obj usersaction.obj vlistbrowserclass.obj \
		hlistbrowserclass.obj datelistbrowserclass.obj invbrowserclass.obj \
		pagedbrowserclass.obj htmlbrowserclass.obj

RECPTOBJECTS = recptmain.obj

LIBRARYOBJECTS = nullproto.obj librarymain.obj

OBJECTS = $(COMMONOBJECTS) $(RECPTOBJECTS) $(LIBRARYOBJECTS)


COLSERVROBJECTS = ..\colservr\mgq.obj \
                  ..\colservr\mgsearch.obj \
                  ..\colservr\querycache.obj \
                  ..\colservr\queryinfo.obj \
                  ..\colservr\collectserver.obj \
                  ..\colservr\colservrconfig.obj \
                  ..\colservr\maptools.obj \
                  ..\colservr\filter.obj \
                  ..\colservr\source.obj \
                  ..\colservr\browsefilter.obj \
                  ..\colservr\queryfilter.obj \
                  ..\colservr\mggdbmsource.obj \
                  ..\colservr\phrasesearch.obj \
                  ..\colservr\phrasequeryfilter.obj

EXEC = recpt.exe library.exe

all: $(EXEC)

clean:
	del *.obj
	del recpt.exe
	del library.exe

install:
	copy library.exe f:\apache\cgi-bin\library.exe

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

RECPT_OBJS = $(COMMONOBJECTS) $(RECPTOBJECTS) \
    ..\..\lib\gsdllib.lib  \
    ..\..\packages\mg\src\text\libtextin.lib \
    ..\..\packages\mg\lib\libmg.lib

recpt.exe: $(RECPT_OBJS)
           $(LINK) $(RECPT_OBJS) $(LIBS) 

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

LIBRARY_OBJS = $(COMMONOBJECTS) $(LIBRARYOBJECTS) $(COLSERVROBJECTS) \
    ..\..\lib\gsdllib.lib \
    ..\..\packages\mg\src\text\libtextin.lib \
    ..\..\packages\mg\lib\libmg.lib

library.exe: $(LIBRARY_OBJS)
	     $(LINK) $(LIBRARY_OBJS) $(LIBS) 
